package org.in2p3.transformation;


import java.util.Iterator;

import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.EnumerationLiteral;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Package;
import org.eclipse.uml2.uml.Parameter;
import org.eclipse.uml2.uml.ParameterDirectionKind;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.State;
import org.eclipse.uml2.uml.StateMachine;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.Vertex;
import org.eclipse.uml2.uml.VisibilityKind;

public class Transformation {
	
	private Class selectedClass = null;
	
	private StateMachine stateMachine = null;
	
	public Transformation(Class currentClass, StateMachine currentStateMachine) {
		selectedClass = currentClass;
		stateMachine = currentStateMachine;
	}
	
	
	/** Applies the transformation pattern of StateMachine */
	public void transform() {
		if(selectedClass!=null && stateMachine!=null) {
		
			/** Create and add StatesEnum type */
			Enumeration type = addStatesEnum();
		
			/** Create and add statesConfiguration property */
			addStatesConfigurationProperty(type);
		
			/** Create and add CheckPreCond() operation */
			addCheckPreCondOperation();
		
			/** Create and add CheckPostCond() operation */
			addCheckPostCondOperation();
		
		}
	}
	
	
	/** Adds the StatesEnum enumeration to selectedClass */
	public Enumeration addStatesEnum() {
		Enumeration statesEnum = Utils.createEnumeration();
		statesEnum.setName("StatesEnum");
		
		Iterator<Vertex> statesIter = stateMachine.getRegions().get(0).getSubvertices().iterator();
		while(statesIter.hasNext()) {
			Vertex currentVertex = statesIter.next();
			if(currentVertex instanceof State) {
				EnumerationLiteral enumLiteral = Utils.createEnumerationLiteral();
				enumLiteral.setName(((State)currentVertex).getName());
				statesEnum.getOwnedLiterals().add(enumLiteral);
			}
		}
		
		selectedClass.getNestedClassifiers().add(statesEnum);
		return statesEnum;
	}
	
	/** Adds the stateConfiguration property to selectedClass */
	public void addStatesConfigurationProperty(Type type) {
		Property property = Utils.createProperty();
		property.setName("stateConfiguration"); 
		property.setType(type);
		property.setVisibility(VisibilityKind.PRIVATE_LITERAL);
		
		selectedClass.getOwnedAttributes().add(property);
	}
	
	/** Adds "EvState checkPreCond(SRCompeting sr)" operation to selectedClass */
	public void addCheckPreCondOperation() {
		Operation checkPreCond = Utils.createOperation();
		checkPreCond.setName("checkPreCond");

		/* Parameters creation */
		Parameter returnParam = Utils.createParameter();
		returnParam.setName("ret");
		returnParam.setDirection(ParameterDirectionKind.RETURN_LITERAL);
		returnParam.setType(getExternType("EvState"));
		
		Parameter inputParam = Utils.createParameter();
		inputParam.setName("sr");
		inputParam.setDirection(ParameterDirectionKind.IN_LITERAL);
		inputParam.setType(getExternType("SRCompeting"));
		
		/* Add parameters to checkPreCond operation */
		checkPreCond.getOwnedParameters().add(inputParam);
		checkPreCond.getOwnedParameters().add(returnParam);
		
		/* Add checkPreCond to the selectedClass */
		selectedClass.getOwnedOperations().add(checkPreCond);
	}
	
	
	/** Adds "void checkPreCond(SRCompeting sr)" operation to selectedClass */
	public void addCheckPostCondOperation() {
		Operation checkPostCond = Utils.createOperation();
		checkPostCond.setName("checkPostCond");
		
		/* Parameters creation */
		Parameter inputParam = Utils.createParameter();
		inputParam.setName("sr");
		inputParam.setDirection(ParameterDirectionKind.IN_LITERAL);
		inputParam.setType(getExternType("SRCompeting"));
		
		/* Add parameters to checkPostCond operation */
		checkPostCond.getOwnedParameters().add(inputParam);
		
		/* Add checkPostCond to the selectedClass */
		selectedClass.getOwnedOperations().add(checkPostCond);		
	}

	
	/** Utility operation that retrieves an external type in the model */
	public Type getExternType(String name) {
		Package externTypesPackage = (Package)selectedClass.getModel().getPackagedElement("ExternTypes");
		NamedElement type = externTypesPackage.getOwnedMember(name);
		if(type instanceof Type)
			return (Type)type;
		return null;
	}
	
}
