package org.in2p3.transformation.popup.actions;

import java.util.Iterator;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IEditorActionDelegate;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.uml2.uml.Behavior;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.StateMachine;
import org.in2p3.transformation.Transformation;

import com.cea.papyrus.core.editpart.IUMLElementEditPart;

public class StateMachineTransformation implements IViewActionDelegate, IActionDelegate, IEditorActionDelegate {

	private Shell shell;
	private Class selectedClass = null;
	
	private final static String message = " Please select a Class with a StateMachine.\n";

	/**
	 * Constructor for Action1.
	 */
	public StateMachineTransformation() {
		super();
	}
	
	public void init(IViewPart view){}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		//shell = targetPart.getSite().getShell();
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		if(selectedClass!=null){
			StateMachine stateMachine = null;

			/** Find associated StateMachine */
			Iterator<Behavior> behaviorsIter = selectedClass.getOwnedBehaviors().iterator();
			while(behaviorsIter.hasNext()) {
				Behavior currentBehavior = behaviorsIter.next();
				if(currentBehavior instanceof StateMachine)
					stateMachine = (StateMachine) currentBehavior;
			}

			if(stateMachine==null) {
				MessageDialog.openError(PlatformUI.getWorkbench().getDisplay().getActiveShell(), "Class "+selectedClass.getName()+" has no associated StateMachine.", message);
			} else {
				Transformation transf = new Transformation(selectedClass, stateMachine);
				transf.transform();
			}	
		}
		
		MessageDialog.openInformation(
				shell,
				"Transformation Plug-in",
		"StateMachine transformation is executed.");
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		action.setEnabled(false);
		//look for selected Element
		Object e=((IStructuredSelection)selection).getFirstElement();
		//this is an element linked to an UML element?
		if (e instanceof IUMLElementEditPart){
			Element element = ((IUMLElementEditPart<Element>)e).getUmlElement();
			//this is a Class?
			if (element instanceof Class){
				action.setEnabled(true);
				selectedClass= (Class)element;
			}
		}
	}

	public void setActiveEditor(IAction action, IEditorPart targetEditor){

	}
}
